import unittest

from . import FixerTestCase


class testFixDivisionTrace(FixerTestCase):
  fixer = "division_trace"

  def test_simple_division(self):
    b = """x / y"""
    a = """division_traced(0,x , y)"""
    self.check(b, a)

  def test_nested_divisions_with_parentheses_1(self):
    b = """(x / y) / z"""
    a = """division_traced(1,(division_traced(0,x , y)) , z)"""
    self.check(b, a)

  def test_nested_divisions_with_parentheses_2(self):
    b = """x / (y / z)"""
    a = """division_traced(1,x , (division_traced(0,y , z)))"""
    self.check(b, a)

  def test_inline_division_1(self):
    b = """1 / 2 / 3"""
    a = """division_traced(1,division_traced(0,1 , 2) , 3)"""
    self.check(b, a)

  def test_inline_division_2(self):
    b = """1 / 2 * 3"""
    a = """division_traced(0,1 , 2) * 3"""
    self.check(b, a)

  def test_inline_division_3(self):
    b = """1 * 2 / 3"""
    a = """division_traced(0,1 * 2 , 3)"""
    self.check(b, a)

  def test_division_on_line_continuation(self):
    b = """x \
  / y"""
    a = """  division_traced(0,x \
  , y)"""
    self.check(b, a)

  def test_multiline_division(self):
    b = """foo = \
  x / y"""
    a = """foo = \
  division_traced(0,x , y)"""
    self.check(b, a)


if __name__ == '__main__':
  unittest.main()
